package examples;

import javax.ejb.*;
import javax.naming.*;
import java.rmi.*;
import javax.rmi.*;
import java.util.*;

/**
 * Sample client code which manipulates a Bank Account Entity Bean.
 */
public class AccountClient {

	public static void main(String[] args) throws Exception {

        // Included so we can send an AccountPK object (just for demonstration)
        System.setSecurityManager(new java.rmi.RMISecurityManager());
        
		Account account = null;

		try {
			/*
			 * Get a reference to the Account Home Object - the
			 * factory for Account EJB Objects
                         */
			Context ctx = new InitialContext(System.getProperties());
			
			Object obj = ctx.lookup("account/AccountHome");
			AccountHome home = (AccountHome) javax.rmi.PortableRemoteObject.narrow(obj, AccountHome.class);

			System.err.println("Total of all accounts in bank initially = " + home.getTotalBankValue());

			/*
			 * Use the factory to create the Account EJB Object
			 */
			Account outro = home.create("123-456-7890", "Adam Smith");

			/*
			 * Find an account
			 */
			Iterator i = home.findByOwnerName("Adam Smith").iterator();
			System.out.println("Contas de Adam Smith: ");
			while (i.hasNext()) {
			  account = (Account) 
			    javax.rmi.PortableRemoteObject.narrow(
			      i.next(), Account.class);
				  System.out.println("Conta " + account.getAccountID() + " tem saldo " + account.getBalance());

			}

			/*
			 * Call the balance() method, and print it
			 */
			System.out.println("Initial Balance = " + account.getBalance());

			/*
			 * Deposit $100 into the account
			 */
			account.deposit(100);

			/*
			 * Retrieve the resulting balance.
			 */
			System.out.println("After depositing 100, account balance = " + account.getBalance());

			System.out.println("Total of all accounts in bank now = " + home.getTotalBankValue());

			/*
			 * Retrieve the Primary Key from the EJB Object
			 */
			AccountPK pk = (AccountPK) account.getPrimaryKey();

			/*
			 * Release our old EJB Object reference.  Now call
			 * find() again, this time querying on Account ID
			 * (i.e. the Primary Key).
			 */
			account = null;
			account = home.findByPrimaryKey(pk);

			/*
			 * Print out current balance
			 */
			System.out.println("Found account with ID " + pk + ".  Balance = " + account.getBalance());

            /*
             * Reading another account. This may cause an exception if account does not exist.
             */
            System.out.println("Looking up another client's account... (may cause exception)");
            Account other = home.findByPrimaryKey(new AccountPK("000-580-33"));
            System.out.println(other.getOwnerName() + " has " + other.getBalance() + " in his acount!");
            
			/*
			 * Try to withdraw $150
			 */
			System.out.println("Now trying to withdraw $1500000, which is more than is currently available.  This should generate an exception..");
			other.withdraw(1500000);
			System.out.println(other.getOwnerName() + " has " + other.getBalance() + " in his acount!");
            

		}
        catch (AccountException e) {
			System.out.println("Caught an account exception: " + e);
		}
		catch (Exception e) {
			System.out.println("Caught exception!");
			e.printStackTrace();
		}
		finally {
			/*
			 * Destroy the Entity permanently
			 */
			try {
				System.out.println("Destroying account.");
				if (account != null) {
					account.remove(); // if you don't do this you won't be able to run the 
                                      // application a second time because of duplicate PKs in insert
				}
			}
			catch (Exception e) {
				e.printStackTrace();
			}
		}
	}
}
